@@ -1,9 +1,9 @@
 use warnings;
 use strict;
 
-package Text::Parser 0.803;
+package Text::Parser 0.900;
 
-# ABSTRACT: Bare text parser, simplifies text parsing.
+# ABSTRACT: Simplifies text parsing. Extensible to parse various grammars.
 
 use Exporter 'import';
 our (@EXPORT_OK) = ();
@@ -12,35 +12,10 @@ our (@EXPORT)    = (@EXPORT_OK);
 
 use Exception::Class (
     'Text::Parser::Exception',
-    'Text::Parser::Exception::ParsingError' => {
+    'Text::Parser::Exception::Constructor' => {
         isa         => 'Text::Parser::Exception',
-        description => 'For all parsing errors',
-        alias       => 'throw_text_parsing_error'
-    },
-    'Text::Parser::Exception::FileNotFound' => {
-        isa         => 'Text::Parser::Exception',
-        description => 'File not found',
-        alias       => 'throw_file_not_found'
-    },
-    'Text::Parser::Exception::FileNotReadable' => {
-        isa         => 'Text::Parser::Exception',
-        description => 'File not readable',
-        alias       => 'throw_file_not_readable'
-    },
-    'Text::Parser::Exception::InvalidFileHandle' => {
-        isa         => 'Text::Parser::Exception',
-        description => 'Bad argument supplied to filehandle()',
-        alias       => 'throw_invalid_filehandle'
-    },
-    'Text::Parser::Exception::InvalidFilename' => {
-        isa         => 'Text::Parser::Exception',
-        description => 'Bad argument supplied to filename()',
-        alias       => 'throw_bad_filename'
-    },
-    'Text::Parser::Exception::FileCantOpen' => {
-        isa         => 'Text::Parser::Exception',
-        description => 'Error opening file',
-        alias       => 'throw_cant_open'
+        description => 'Bad arguments to new()',
+        alias       => 'throw_new'
     },
     'Text::Parser::Exception::BadReadInput' => {
         isa => 'Text::Parser::Exception',
@@ -50,243 +25,239 @@ use Exception::Class (
     },
 );
 
+use Moose;
+use MooseX::CoverableModifiers;
+use namespace::autoclean;
+use FileHandle;
 use Try::Tiny;
-use Scalar::Util 'openhandle';
-use Role::Tiny;
-use IO::Select;
+use Moose::Util 'apply_all_roles';
+use Moose::Util::TypeConstraints;
+
+coerce FileHandle => from GlobRef =>
+    via { FileHandle->new_from_fd( $$_, 'r' ) };
+
+subtype FileReadable => as
+    Any => where { not defined $_ or ( $_ and -f $_ and -r $_ ) },
+    message {"\'$_\' is not a valid readable file"};
+
+enum MultilineType => [qw(join_next join_last)];
+
+no Moose::Util::TypeConstraints;
 
 
-sub new {
-    my $pkg = shift;
-    return if not __check_options(@_);
-    my $hash = __set_options(@_);
-    my $obj  = bless { __options => $hash }, $pkg;
-    return $obj->__return_my_object();
-}
+my %allowed_args = ( auto_chomp => 1, multiline_type => 1 );
 
-my (%allowed_options)
-    = ( auto_chomp => '0|1', multiline_type => 'join_next|join_last' );
-my (%default_values) = ( auto_chomp => 0, multiline_type => undef );
-
-sub __check_options {
-    my (%opt) = @_;
-    foreach my $k ( keys %opt ) {
-        return 0 if not exists $allowed_options{$k};
-        my (@allowed) = split /\s*[|]\s*/, $allowed_options{$k};
-        return 0 if not grep { $_ eq $opt{$k} } @allowed;
+before BUILDARGS => sub {
+    my $class = shift;
+    throw_new error => 'Arguments to new() must be a hash' if @_ % 2;
+    my (%hash) = @_;
+    foreach my $k ( keys %hash ) {
+        throw_new error => "Invalid key $k as input to new()"
+            if not exists $allowed_args{$k};
     }
-    return 1;
+};
+
+sub BUILD {
+    my $self = shift;
+    return if not $self->_has_mult_type or not defined $self->multiline_type;
+    apply_all_roles $self, 'Text::Parser::Multiline';
 }
 
-sub __set_options {
-    my (%opt) = @_;
-    foreach my $k ( keys %default_values ) {
-        $opt{$k} = $default_values{$k} if not exists $opt{$k};
-    }
-    return \%opt;
-}
 
-sub __return_my_object {
-    my $obj = shift;
-    return $obj if not defined $obj->setting('multiline_type');
-    Role::Tiny->apply_roles_to_object( $obj, 'Text::Parser::Multiline' );
-    return $obj;
-}
+has multiline_type => (
+    is        => 'ro',
+    isa       => 'MultilineType|Undef',
+    lazy      => 1,
+    default   => undef,
+    predicate => '_has_mult_type',
+);
+
+
+has auto_chomp => (
+    is      => 'ro',
+    isa     => 'Bool',
+    lazy    => 1,
+    default => 0,
+);
 
 
 sub setting {
-    my ( $self, $key ) = ( shift, shift );
-    return if not defined $key or not exists $self->{__options}{$key};
-    return $self->{__options}{$key};
+    my $self = shift;
+    return if not @_;
+    my $setting = shift;
+    my %allowed = ( multiline_type => 1, auto_chomp => 1 );
+    return if not exists $allowed{$setting};
+    return $self->$setting();
 }
 
 
+
+has abort => (
+    is      => 'rw',
+    isa     => 'Bool',
+    lazy    => 1,
+    default => 0,
+    traits  => ['Bool'],
+    reader  => 'has_aborted',
+    handles => {
+        abort_reading => 'set',
+        _clear_abort  => 'unset'
+    },
+);
+
+
 sub read {
-    my ( $self, $input ) = @_;
-    return if not $self->__is_file_known_or_opened($input);
-    $self->__store_check_input($input);
-    $self->__read_and_close_filehandle();
+    my $self = shift;
+    return if not defined $self->_handle_read_inp(@_);
+    $self->__read_and_close_filehandle;
 }
 
-sub __store_check_input {
-    my ( $self, $input ) = @_;
-    return                           if not defined $input;
-    return $self->filename($input)   if ref($input) eq '';
-    return $self->filehandle($input) if ref($input) eq 'GLOB';
-    __throw_bad_input_to_read( ref($input) );
+sub _handle_read_inp {
+    my $self = shift;
+    return $self->filehandle if not @_;
+    my $inp = shift;
+    return if not ref($inp) and not $inp;
+    return $self->__save_file_handle($inp);
 }
 
-sub __throw_bad_input_to_read {
-    throw_bad_input_to_read error => 'Unexpected ' . shift
-        . ' type input to read() ; must be either string filename or GLOB';
-}
-
-sub __is_file_known_or_opened {
-    my ( $self, $fname ) = @_;
-    return 0 if not defined $fname and not exists $self->{__filehandle};
-    return 0 if defined $fname and not $fname;
-    return 1;
+sub __save_file_handle {
+    my ( $self, $inp ) = ( shift, shift );
+    return $self->filename($inp) if not ref($inp);
+    return $self->filehandle($inp)
+        if ref($inp) eq 'GLOB'
+        or ( defined blessed($inp) and blessed($inp) eq 'FileHandle' );
+    throw_bad_input_to_read error => "$inp is an unknown type of input";
 }
 
 sub __read_and_close_filehandle {
     my $self = shift;
-    $self->__init_read_fh;
+    $self->_reset_line_count;
+    $self->_empty_records;
+    $self->_clear_abort;
     $self->__read_file_handle;
-    $self->__close_file;
-}
-
-sub __init_read_fh {
-    my $self = shift;
-    $self->lines_parsed(0);
-    $self->{__bytes_read} = 0;
-    delete $self->{__records} if exists $self->{__records};
-    delete $self->{__abort_reading};
+    $self->_close_filehandles if $self->_has_filename;
 }
 
 sub __read_file_handle {
     my $self = shift;
     my $fh   = $self->filehandle();
     while (<$fh>) {
-        last if not $self->__parse_line_and_next($_);
+        last if not $self->__parse_line($_);
     }
 }
 
-sub __parse_line_and_next {
+sub __parse_line {
     my ( $self, $line ) = ( shift, shift );
-    $self->lines_parsed( $self->lines_parsed + 1 );
-    chomp $line if $self->setting('auto_chomp');
+    $self->_next_line_parsed();
+    chomp $line if $self->auto_chomp;
     $self->__try_to_parse($line);
-    return not exists $self->{__abort_reading};
+    return not $self->has_aborted;
 }
 
 sub __try_to_parse {
     my ( $self, $line ) = @_;
     try { $self->save_record($line); }
     catch {
-        $self->__close_file;
+        $self->_close_filehandles;
         $_->rethrow;
     };
 }
 
-sub __close_file {
+
+has filename => (
+    is        => 'rw',
+    isa       => 'FileReadable|Undef',
+    lazy      => 1,
+    init_arg  => undef,
+    default   => undef,
+    predicate => '_has_filename',
+    clearer   => '_clear_filename',
+    trigger   => \&_set_filehandle,
+);
+
+sub _set_filehandle {
     my $self = shift;
-    return if not exists $self->{__filename};
-    close $self->{__filehandle};
-    delete $self->{__filehandle};
+    my $fh   = FileHandle->new( $self->filename, 'r' );
+    return $self->_save_filehandle($fh);
 }
 
 
-sub filename {
-    my $self = shift;
-    $self->__open_file( $self->__is_readable_file(shift) ) if scalar(@_);
-    return ( exists $self->{__filename} ) ? $self->{__filename} : undef;
-}
-
-sub __is_readable_file {
-    my ( $self, $fname ) = @_;
-    throw_bad_filename( error => "$fname is not a string" )
-        if ref($fname) ne '';
-    throw_file_not_found( error => "$fname is not a file" )
-        if not -f $fname;
-    throw_file_not_readable( error => "$fname is not readable" )
-        if not -r $fname;
-    return $fname;
-}
-
-sub __open_file {
-    my ( $self, $fname ) = @_;
-    open my $fh, '<', $fname
-        or throw_cant_open( error => "Error while opening file $fname" );
-    $self->__close_file if exists $self->{__filehandle};
-    $self->{__filename}   = $fname;
-    $self->{__filehandle} = $fh;
-}
-
+has filehandle => (
+    is        => 'rw',
+    isa       => 'FileHandle|Undef',
+    lazy      => 1,
+    init_arg  => undef,
+    default   => undef,
+    predicate => '_has_filehandle',
+    writer    => '_save_filehandle',
+    reader    => '_get_filehandle',
+    clearer   => '_close_filehandles',
+);
 
 sub filehandle {
-    my ( $self, $fhref ) = @_;
-    $self->__save_file_handle($fhref) if $self->__check_file_handle($fhref);
-    return ( exists $self->{__filehandle} ) ? $self->{__filehandle} : undef;
-}
-
-sub __save_file_handle {
-    my ( $self, $fhref ) = @_;
-    $self->{__filehandle} = $$fhref;
-    delete $self->{__filename} if exists $self->{__filename};
-    $self->{__size} = ( stat $$fhref )[7];
-}
-
-sub __check_file_handle {
-    my ( $self, $fhref ) = @_;
-    return 0 if not defined $fhref;
-    $self->_invalid_filehandle($fhref);
-    return $self->_fh_is_readable($fhref);
-}
-
-sub _invalid_filehandle {
-    my ( $self, $fhref ) = ( shift, shift );
-    throw_invalid_filehandle( error => "$fhref is not a valid filehandle" )
-        if ref($fhref) ne 'GLOB';
-    throw_file_not_readable( error => "$$fhref is a closed filehandle" )
-        if not defined openhandle($fhref);
-}
-
-sub _fh_is_readable {
-    my ( $self, $fhref ) = ( shift, shift );
-    throw_file_not_readable(
-        error => "The filehandle $$fhref is not readable" )
-        if not -r $fhref;
-    return 1;
-}
-
-
-sub lines_parsed {
     my $self = shift;
-    return $self->{__current_line} = shift if @_;
-    return ( exists $self->{__current_line} ) ? $self->{__current_line} : 0;
+    return if not @_ and not $self->_has_filehandle;
+    $self->_save_filehandle(@_) if @_;
+    $self->_clear_filename if @_;
+    my $fh = $self->_get_filehandle;
+    return $fh;
 }
 
 
+has lines_parsed => (
+    is       => 'ro',
+    isa      => 'Int',
+    lazy     => 1,
+    init_arg => undef,
+    default  => 0,
+    traits   => ['Counter'],
+    handles  => {
+        _next_line_parsed => 'inc',
+        _reset_line_count => 'reset',
+    }
+);
+
+
 sub save_record {
-    my $self = shift;
-    $self->{__records} = [] if not defined $self->{__records};
-    push @{ $self->{__records} }, shift;
+    my ( $self, $record ) = ( shift, shift );
+    $self->push_records($record);
 }
 
 
-sub abort_reading {
-    my $self = shift;
-    return $self->{__abort_reading} = 1;
-}
 
 
-sub get_records {
-    my $self = shift;
-    return () if not exists $self->{__records};
-    return @{ $self->{__records} };
-}
+has records => (
+    isa        => 'ArrayRef[Any]',
+    is         => 'ro',
+    lazy       => 1,
+    default    => sub { return []; },
+    auto_deref => 1,
+    init_arg   => undef,
+    traits     => ['Array'],
+    handles    => {
+        get_records    => 'elements',
+        push_records   => 'push',
+        pop_record     => 'pop',
+        _empty_records => 'clear',
+        _num_records   => 'count',
+        _access_record => 'accessor',
+    },
+);
 
 
 sub last_record {
-    my $self = shift;
-    return if not exists $self->{__records};
-    my (@record) = @{ $self->{__records} };
-    return $record[$#record];
-}
-
-
-sub pop_record {
-    my $self = shift;
-    return if not exists $self->{__records};
-    pop @{ $self->{__records} };
+    my $self  = shift;
+    my $count = $self->_num_records();
+    return if not $count;
+    return $self->_access_record( $count - 1 );
 }
 
 
 sub is_line_continued {
     my $self = shift;
-    return 0 if not defined $self->setting('multiline_type');
+    return 0 if not defined $self->multiline_type;
     return 0
-        if $self->setting('multiline_type') eq 'join_last'
+        if $self->multiline_type eq 'join_last'
         and $self->lines_parsed() == 1;
     return 1;
 }
@@ -299,12 +270,9 @@ sub join_last_line {
 }
 
 
-sub has_aborted {
-    my $self = shift;
-    return $self->{__abort_reading} if exists $self->{__abort_reading};
-    return 0;
-}
+__PACKAGE__->meta->make_immutable;
 
+no Moose;
 
 1;
 
@@ -316,11 +284,11 @@ __END__
 
 =head1 NAME
 
-Text::Parser - Bare text parser, simplifies text parsing.
+Text::Parser - Simplifies text parsing. Extensible to parse various grammars.
 
 =head1 VERSION
 
-version 0.803
+version 0.900
 
 =head1 SYNOPSIS
 
@@ -334,23 +302,21 @@ The above code reads the first command-line argument as a string, and assuming i
 
 =head1 RATIONALE
 
-Text parsing is perhaps the single most common thing that almost every program does. Yet we don't have a lean, flexible, file parser, where the programmer only has to specify the grammar/syntax of the text input.
-
-A simple text file parser should have to only specify the "grammar" it intends to interpret. Everything else, like C<open>ing a file handle, C<close>ing the file handle, tracking line-count, joining continued lines into one, reporting any errors in line continuation, etc., distract from what should be the main goal : parsing for data records from the text input. Unfortunately, most programmers have to continue writing code that calls C<open>, C<close>, etc., and keep track of things that should have been simple features of every text file parser. And if they have to read a second file with a different grammar, usually, all these calls to C<open>, C<close>, etc. are repeated.
+Text parsing is perhaps the single most common thing that almost every Perl program does. Yet we don't have a lean, flexible, text parser. A simple text parser should have to only specify the "grammar" it intends to interpret and collect records from. Everything else, like C<open>ing a file handle, C<close>ing the file handle, tracking line-count, joining continued lines into one, reporting any errors in line continuation, etc., distract from the main goal : getting data records from the text input. Unfortunately, most programmers have to continue writing code that should just have been simple features of every text file parser. And if they have to read a second file with a different grammar, usually, all that code needs to be repeated.
 
 This class automates all "mundane" operations like C<open>, C<close>, line-count, and storage/deletion/retrieval of records, joining continued lines, etc. You don't have to bother with a lot of book-keeping when you write your parser. You focus on specifying a grammar and telling the parser what information to store. To do this, you just inherit this class and override one method (C<L<save_record|/save_record>>). And voila! you have a parser. Everything else is taken care of by this class. Look at L<these examples|/EXAMPLES> to see how easy this can be. If your text format allows a single line to be split into several lines with a continuation character, you again need to specify only a few additional things, and everything else is taken care of for you. See L<these examples|/"Example 4 : Multi-line parsing">.
 
 =head1 DESCRIPTION
 
-C<Text::Parser> is a bare-bones text parsing class. It is ignorant of the text format, and cannot recognize any grammars, but derived classes that inherit from it can specify this. They can do this usually by overriding just one of the methods in this class.
+C<Text::Parser> is a bare-bones text parsing class. It is ignorant of the text format, and cannot recognize any grammars, but derived classes that inherit from it can specify this. They can do this usually by overriding just one of the methods in this class. Of course derived classes can create any additional attributes and methods they need to do their task of extracting information records out of each line.
 
-Future versions are expected to include progress-bar support, parsing text from sockets, or a chunk of memory. All these software features are text-format independent and can be re-used in parsing any text format. Derived classes of C<Text::Parser> will be able to take advantage of these features.
+Future versions are expected to include progress-bar support, parsing text from sockets, UTF support, or parsing from a chunk of memory. All these software features are text-format independent and can be re-used in parsing any text format. Derived classes of C<Text::Parser> will be able to take advantage of these features seamlessly and only have to focus on extracting information, while the base class handles the "mundane" parts.
 
 =head1 METHODS
 
 =head2 new
 
-Constructor. Takes options in the form of a hash. You can thus create an object of a parser like this.
+Constructor. Takes options in the form of a hash. Throws an exception if you use wrong inputs to create an object. You can thus create an object of a parser like this.
 
     my $parser = Text::Parser->new(
         auto_chomp => 0,               # 0 (Default) or 1 - automatically chomp lines
@@ -372,27 +338,44 @@ If your format allows something like a trailing back-slash or some other charact
 
 =item *
 
-If your format allows some character to indicate that text on the current line is part of the I<B<last>> line, then choose C<join_last>. See L<this simple SPICE line-joiner|/"Simple SPICE line joiner"> as an example.
-
-=item *
-
-If you have no continuation character, but you want to just join all the lines into one single line and then call C<save_record> only once for the whole text block, then use C<join_last>. See L<this trivial line-joiner|/"Trivial line-joiner">.
+If your format allows some character to indicate that text on the current line is part of the I<B<last>> line, then choose C<join_last>. See L<this simple SPICE line-joiner|/"Simple SPICE line joiner"> as an example. B<Note:> If you have no continuation character, but you want to just join all the lines into one single line and then call C<save_record> only once for the whole text block, then use C<join_last>. See L<this trivial line-joiner|/"Trivial line-joiner">.
 
 =back
 
+=head2 multiline_type
+
+This method replaces the older C<setting> method. It is a read-only method and returns the value of the C<multiline_type> attribute.
+
+    my $mult = $parser->multiline_type;
+    print "Parser is a multi-line parser of type: $mult" if defined $mult;
+
+=head2 auto_chomp
+
+This method replaces the older C<setting> method. It is a read-only method and returns the status of the C<auto_chomp> attribute.
+
+    print "Parser will chomp lines automatically\n" if $parser->auto_chomp;
+
 =head2 setting
 
-Takes a single string as argument, and returns the value of that setting. The string must be one of C<auto_chomp>, or C<multiline_type>.
+B<I<Deprecated>>
 
-    print "Parser will chomp\n" if $parser->setting('auto_chomp');
-    my $mult = $parser->setting('multiline_type');
-    print "Parser is a multi-line parser of type: $mult\n" if defined $mult;
+This method has been deprecated. Use C<multiline_type> and C<auto_chomp> instead.
 
-These settings are set during the parser construction.
+=head2 abort_reading
+
+Takes no arguments. Returns C<1>. You will probably never call this method in your main program.
+
+This method is usually used only in the derived class. See L<this example|/"Example 3 : Aborting without errors">.
+
+=head2 has_aborted
+
+Takes no arguments, returns a boolean to indicate if text reading was aborted in the middle.
+
+    print "Aborted\n" if $parser->has_aborted();
 
 =head2 read
 
-Takes zero or one argument which could be a string containing the name of the file, or a filehandle reference (a C<GLOB>) like C<\*STDIN>. Throws an exception if filename/C<GLOB> provided is either non-existent or cannot be read for any reason.
+Takes zero or one argument which could be a string containing the name of the file, or a filehandle reference (a C<GLOB>) like C<\*STDIN> or an object of the C<L<FileHandle>> class. Throws an exception if filename/C<GLOB> provided is either non-existent or cannot be read for any reason.
 
 B<Note:> Normally if you provide the C<GLOB> of a file opened for write, some Operating Systems allow reading from it too, and some don't. Read the documentation for C<L<filehandle|/filehandle>> for more on this.
 
@@ -408,7 +391,7 @@ B<Note:> Normally if you provide the C<GLOB> of a file opened for write, some Op
 
 Returns once all records have been read or if an exception is thrown for any parsing errors, or if reading has been aborted with the C<L<abort_reading|/abort_reading>> method.
 
-If you provide a string file name as input, the function will handle all C<open> and C<close> operations on files even if any exception is thrown, or if the reading has been aborted. But if you pass a file handle C<GLOB> instead, then the file handle won't be closed and it will be the responsibility of the calling program to close the filehandle.
+If you provide a filename as input, the function will handle all C<open> and C<close> operations on files even if any exception is thrown, or if the reading has been aborted. But if you pass a file handle C<GLOB> or C<FileHandle> object instead, then the file handle won't be closed and it will be the responsibility of the calling program to close the filehandle.
 
     $parser->read('myfile.txt');
     # Will handle open, parsing, and closing of file automatically.
@@ -418,7 +401,7 @@ If you provide a string file name as input, the function will handle all C<open>
     # Will not close MYFH and it is the respo
     close MYFH;
 
-When you do read a new file or input stream with this method, you will lose all the records stored from the previous read operation. So this means that if you want to read a different file with the same parser object, (unless you don't care about the records from the last file you read) you should use the C<L<get_records|/get_records>> method to retrieve all the read records before parsing a new file. So all those calls to C<read> in the example above were parsing three different files, and each successive call overwrote the records from the previous call.
+When you do read a new file name or file handle with this method, you will lose all the records stored from the previous read operation. So this means that if you want to read a different file with the same parser object, (unless you don't care about the records from the last file you read) you should use the C<L<get_records|/get_records>> method to retrieve all the read records before parsing a new file. So all those calls to C<read> in the example above were parsing three different files, and each successive call overwrote the records from the previous call.
 
     $parser->read($file1);
     my (@records) = $parser->get_records();
@@ -426,7 +409,7 @@ When you do read a new file or input stream with this method, you will lose all
     $parser->read(\*STDIN);
     my (@stdin) = $parser->get_records();
 
-B<Inheritance Recommendation:> To extend the class to other file formats, override C<L<save_record|/save_record>> instead of this one.
+B<Note:> To extend the class to other file formats, override C<L<save_record|/save_record>> instead of this one.
 
 =head3 Future Enhancement
 
@@ -454,17 +437,10 @@ But if you do a C<read> with a filehandle as argument, you'll see that the last
 
 =head2 filehandle
 
-Takes zero or one C<GLOB> argument and saves it for future a C<L<read|/read>> call. Returns the filehandle last saved, or C<undef> if none was saved. Remember that after a successful C<read> call, filehandles are lost.
+Takes zero or one argument that must be either a filehandle C<GLOB> (such as C<\*STDIN>) or an object of the C<FileHandle> class. The method saves it for future a C<L<read|/read>> call. Returns the filehandle last saved, or C<undef> if none was saved. Remember that after a successful C<read> call, filehandles are lost.
 
     my $fh = $parser->filehandle();
 
-B<Note:> As such there is a check to ensure one is not supplying a write-only filehandle. For example, if you specify the filehandle of a write-only file or if the file is opened for write and you cannot read from it. The weird thing is that some of the standard filehandles like C<STDOUT> don't behave uniformly across all platforms. On most POSIX platforms, C<STDOUT> is readable. On such platforms you will not get any exceptions if you try to do this:
-
-    $parser->filehandle(\*STDOUT);  ## Works on many POSIX platforms
-                                    ## Throws exception on others
-
-Again, it looks like on Win32 systems C<-r STDIN> returns C<0>. If someone knows how to fix these, please L<let me know|/BUGS>.
-
 Like in the case of C<L<filename|/filename>> method, if after you C<read> with a filehandle, you call C<read> again, this time with a file name, the last filehandle is lost.
 
     my $lastfh = $parser->filehandle();
@@ -492,12 +468,6 @@ In an application that uses a text parser, you will most-likely never call this
 
 Derived classes can decide to store records in a different form. A derived class could, for example, store the records in the form of hash references (so that when you use C<L<get_records|/get_records>>, you'd get an array of hashes), or maybe even another array reference (so when you use C<get_records> you'd get an array of arrays). The L<CSV parser example|/"Example 1 : A simple CSV Parser"> does the latter.
 
-=head2 abort_reading
-
-Takes no arguments. Returns C<1>. You will probably never call this method in your main program.
-
-This method is usually used only in the derived class. See L<this example|/"Example 3 : Aborting without errors">.
-
 =head2 get_records
 
 Takes no arguments. Returns an array containing all the records saved by the parser.
@@ -507,11 +477,13 @@ Takes no arguments. Returns an array containing all the records saved by the par
         print "Record: $i: ", $record, "\n";
     }
 
-=head2 last_record
+=head2 push_records
 
-Takes no arguments and returns the last saved record. Leaves the saved records untouched.
+Don't override this method unless you know what you're doing. This method is useful if you have to copy the records from another parser. It is a general-purpose method for storing records that prepared before-hand. It is not supposed to be used to modify the arguments and make records (like C<L<save_record|/save_record>> does).
 
-    my $last_rec = $parser->last_record;
+    $parser->push_records(
+        $another_parser->get_records
+    );
 
 =head2 pop_record
 
@@ -521,6 +493,12 @@ Takes no arguments and pops the last saved record.
     $uc_last = uc $last_rec;
     $parser->save_record($uc_last);
 
+=head2 last_record
+
+Takes no arguments and returns the last saved record. Leaves the saved records untouched.
+
+    my $last_rec = $parser->last_record;
+
 =head2 is_line_continued
 
 Takes a string argument. The default method provided will return C<0> if the parser is not a multi-line parser. If it is a multi-line parser, return value depends on the type of multiline parser. 
@@ -533,16 +511,10 @@ But if it is of type C<'join_next'>, then it returns C<1> for all lines uncondit
 
 =head2 join_last_line
 
-This method is used in multi-line text parsing. The method takes two string arguments. The default implementation just concatenates two strings and returns the result. You should redefine this method to strip any continuation characters and join the strings with any required spaces etc.
+This method can be overridden in multi-line text parsing. The method takes two string arguments and joins them in a way that removes the continuation character. The default implementation just concatenates two strings and returns the result without removing anything. You should redefine this method to strip any continuation characters and join the strings with any required spaces etc.
 
     $parser->join_last_line('last line', ' + this line');
 
-=head2 has_aborted
-
-Takes no arguments, returns a boolean to indicate if text reading was aborted in the middle.
-
-    print "Aborted\n" if $parser->has_aborted();
-
 =head1 EXAMPLES
 
 The following examples should illustrate the use of inheritance to parse various types of text file formats.
@@ -553,6 +525,8 @@ Derived classes simply need to override one method : C<L<save_record|/save_recor
 
 Notice that the creation of a data structure is not the objective of a parser. It is simply concerned with collecting data and arranging it in a form that can be used. That's all. Data structures can be created by a different part of your program using the data collected by your parser.
 
+B<Note:> There is support for L<Moose>. So you could use C<extends 'Text::Parser'> instead of the C<use parent> pragma in these examples. The examples in this documentation will show non-L<Moose> classic Perl OO derived classes for ease of understanding. Those who know how to C<use> class automators like L<Moo>/L<Moose> should be able to follow.
+
 =head2 Example 1 : A simple CSV Parser
 
 We will write a parser for a simple CSV file that reads each line and stores the records as array references. This example is oversimplified, and does B<not> handle embedded newlines.
@@ -769,6 +743,18 @@ Try this parser with a SPICE deck with continuation characters and see what you
 
 L<Text::Parser::Multiline>
 
+=item *
+
+L<FileHandle>
+
+=item *
+
+L<Moose>
+
+=item *
+
+L<Text::CSV>
+
 =back
 
 =head1 BUGS
